;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Nintendo 64 system include file
;; Part of the naken_asm assembler
;;
;; Generated by: Michael Kohn (mike@mikekohn.net)
;;         From: RSP Programmer's Guide
;;         Date: 2021-Mar-14
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; KUSEG: TLB mapped.
;; KSEG0: Direct mapped, cached.
;; KSEG1: Direct mapped, uncached.
;; KSSEG: TLB mapped.
;; KSEG3: TLB mapped.
KUSEG equ 0x0000_0000
KSEG0 equ 0x8000_0000
KSEG1 equ 0xa000_0000
KSSEG equ 0xc000_0000
KSEG3 equ 0xe000_0000

PI_BASE   equ 0xa460_0000

PIF_BASE   equ 0xbfc0_0000
PIF_ROM    equ 0x0000
PIF_RAM    equ 0x07c0

DMA_RAM_ADDRESS           equ 0x0000
DMA_CART_ADDRESS          equ 0x0004
DMA_TRANSFER_SIZE_TO_CART equ 0x0008
DMA_TRANSFER_SIZE_TO_RAM  equ 0x000C
DMA_STATUS                equ 0x0010

DMA_BUSY  equ 0x0000_0001
DMA_ERROR equ 0x0000_0008

;; MIPS VR4300 MMU Coprocesser 

CP0_INDEX        equ $0  ;; Programmable pointer into TLB array
CP0_RANDOM       equ $1  ;; Pseudorandom pointer into TLB array (read only)
CP0_ENTRY_LO_0   equ $2  ;; Low half of TLB entry for even virtual address (VPN)
CP0_ENTRY_LO_1   equ $3  ;; Low half of TLB entry for odd virtual address (VPN)
CP0_CONTEXT      equ $4  ;; Pointer to Kernel virtual PTE table in 32-bit mode
CP0_PAGE_MASK    equ $5  ;; Page size specification
CP0_WIRED        equ $6  ;; Number of wired TLB entries
CP0_RESERVED_0   equ $7
CP0_BAD_VADDR    equ $8  ;; Display of virt address that occurred an error last
CP0_COUNT        equ $9  ;; Timer Count
CP0_ENTRY_HI     equ $10 ;; High half of TLB entry (including ASID)
CP0_COMPARE      equ $11 ;; Timer Compare Value
CP0_STATUS       equ $12 ;; Operation status setting
CP0_CAUSE        equ $13 ;; Display of cause of last exception
CP0_EPC          equ $14 ;; Exception Program Counter
CP0_PRID         equ $15 ;; Processor Revision Identifie
CP0_CONFIG       equ $16 ;; Memory system mode setting
CP0_LL_ADDR      equ $17 ;; Load Linked instruction address display
CP0_WATCH_LO     equ $18 ;; Memory reference trap address low bits
CP0_WATCH_HI     equ $19 ;; Memory reference trap address high bits
CP0_XCONTEXT     equ $20 ;; Pointer to Kernel virtual PTE table in 64-bit mode
CP0_RESERVED_1   equ $21
CP0_RESERVED_2   equ $22
CP0_RESERVED_3   equ $23
CP0_RESERVED_4   equ $24
CP0_RESERVED_5   equ $25
CP0_PARITY_ERROR equ $26 ;; Cache parity bits
CP0_CACHE_ERROR  equ $27 ;; Cache Error and Status register
CP0_TAG_LO       equ $28 ;; Cache Tag register low
CP0_TAG_HI       equ $29 ;; Cache Tag register high
CP0_ERROR_EPC    equ $30 ;; Error Exception Program Counter
CP0_RESERVED_6   equ $31

