/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2020 by Michael Kohn
 *
 */

#include <stdlib.h>

#include "table/propeller2.h"

struct _table_propeller2 table_propeller2[] =
{
  { "nop",     0x00000000, 0xffffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "ror",     0x00000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "rol",     0x00200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "shr",     0x00400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "shl",     0x00600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "rcr",     0x00800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "rcl",     0x00a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sar",     0x00c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sal",     0x00e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "add",     0x01000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "addx",    0x01200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "adds",    0x01400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "addsx",   0x01600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sub",     0x01800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "subx",    0x01a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "subs",    0x01c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "subsx",   0x01e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmp",     0x02000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmpx",    0x02200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmps",    0x02400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmpsx",   0x02600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmpr",    0x02800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmpm",    0x02a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "subr",    0x02c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cmpsub",  0x02e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fge",     0x03000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fle",     0x03200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fges",    0x03400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fles",    0x03600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sumc",    0x03800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sumnc",   0x03a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sumz",    0x03c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "sumnz",   0x03e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "testb",   0x04000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "testbn",  0x04200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "testb",   0x04400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 1, 2, 2, 2, 2, 2, 2 },
  { "testbn",  0x04600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 1, 2, 2, 2, 2, 2, 2 },
  { "testb",   0x04800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 2, 2, 2, 2, 2, 2, 2 },
  { "testbn",  0x04a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 2, 2, 2, 2, 2, 2, 2 },
  { "testb",   0x04c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 3, 2, 2, 2, 2, 2, 2 },
  { "testbn",  0x04e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 0, 3, 2, 2, 2, 2, 2, 2 },
  { "bitl",    0x04000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bith",    0x04200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitc",    0x04400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitnc",   0x04600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitz",    0x04800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitnz",   0x04a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitrnd",  0x04c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "bitnot",  0x04e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "and",     0x05000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "andn",    0x05200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "or",      0x05400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "xor",     0x05600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "muxc",    0x05800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "muxnc",   0x05a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "muxz",    0x05c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "muxnz",   0x05e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "mov",     0x06000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "not",     0x06200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "not",     0x06200000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "abs",     0x06400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "abs",     0x06400000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "neg",     0x06600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "neg",     0x06600000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negc",    0x06800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negc",    0x06800000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negnc",   0x06a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negnc",   0x06a00000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negz",    0x06c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negz",    0x06c00000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negnz",   0x06e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "negnz",   0x06e00000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "incmod",  0x07000000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "decmod",  0x07200000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "zerox",   0x07400000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "signx",   0x07600000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "encod",   0x07800000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "encod",   0x07800000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "ones",    0x07a00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "ones",    0x07a00000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "test",    0x07c00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "test",    0x07c00000, 0x0fe40000, 1, { OP_D,      OP_D,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "testn",   0x07e00000, 0x0fe00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "setnib",  0x08000000, 0x0fc00000, 3, { OP_D,      OP_NUM_S,  OP_N_3  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setnib",  0x08000000, 0x0ffbfe00, 1, { OP_NUM_S,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getnib",  0x08400000, 0x0fc00000, 3, { OP_D,      OP_NUM_S,  OP_N_3  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getnib",  0x08400000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolnib",  0x08800000, 0x0fc00000, 3, { OP_D,      OP_NUM_S,  OP_N_3  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolnib",  0x08800000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setbyte", 0x08c00000, 0x0fe00000, 3, { OP_D,      OP_NUM_S,  OP_N_2  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setbyte", 0x08c00000, 0x0ffbfe00, 1, { OP_NUM_S,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getbyte", 0x08e00000, 0x0fe00000, 3, { OP_D,      OP_NUM_S,  OP_N_2  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getbyte", 0x08e00000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolbyte", 0x09000000, 0x0fe00000, 3, { OP_D,      OP_NUM_S,  OP_N_2  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolbyte", 0x09000000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setword", 0x09200000, 0x0ff00000, 3, { OP_D,      OP_NUM_S,  OP_N_1  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setword", 0x09200000, 0x0ffbfe00, 1, { OP_NUM_S,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getword", 0x09300000, 0x0ff00000, 3, { OP_D,      OP_NUM_S,  OP_N_1  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getword", 0x09300000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolword", 0x09400000, 0x0ff00000, 3, { OP_D,      OP_NUM_S,  OP_N_1  }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rolword", 0x09400000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsn",   0x09500000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsn",   0x09540000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgn",   0x09580000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgn",   0x095c0000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsb",   0x09600000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsb",   0x09640000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgb",   0x09680000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgb",   0x096c0000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsw",   0x09700000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altsw",   0x09740000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgw",   0x09780000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altgw",   0x097c0000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altr",    0x09800000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altr",    0x09840000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altd",    0x09880000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altd",    0x098c0000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "alts",    0x09900000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "alts",    0x09940000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altb",    0x09980000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "altb",    0x099c0000, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "alti",    0x09a00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "alti",    0x09a40164, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setr",    0x09a80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setd",    0x09b00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "sets",    0x09b80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "decod",   0x09c00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "decod",   0x09c00000, 0x0ffc0000, 1, { OP_D,      OP_D,      OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "bmask",   0x09c80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "bmask",   0x09c80000, 0x0ffc0000, 1, { OP_D,      OP_D,      OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "crcbit",  0x09d00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "crcnib",  0x09d80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "muxnits", 0x09e00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "muxnibs", 0x09e80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "muxq",    0x09f00000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "movbyts", 0x09f80000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "mul",     0x0a000000, 0x0ff00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "muls",    0x0a100000, 0x0ff00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "sca",     0x0a200000, 0x0ff00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "scas",    0x0a300000, 0x0ff00000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 1, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "addpix",  0x0a400000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7 },
  { "mulpix",  0x0a480000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7 },
  { "blnpix",  0x0a500000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7 },
  { "mixpix",  0x0a580000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7 },
  { "addct1",  0x0a600000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "addct2",  0x0a680000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "addct3",  0x0a700000, 0x0ff80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wmlong",  0x0a780000, 0x0ff80000, 2, { OP_D,      OP_NUM_SP, OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "rqpin",   0x0a800000, 0x0fe80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rdpin",   0x0a880000, 0x0fe80000, 2, { OP_D,      OP_NUM_S,  OP_NONE }, 0, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rdlut",   0x0aa00000, 0x0fe00000, 2, { OP_D,      OP_NUM_SP, OP_NONE }, 1, 1, 1, 0, 3, 3, 3, 3, 3, 3 },
  { "rdbyte",  0x0ac00000, 0x0fe00000, 2, { OP_D,      OP_NUM_SP, OP_NONE }, 1, 1, 1, 0, 9, 16, 9, 26, 9, 24 },
  { "rdword",  0x0ae00000, 0x0fe00000, 2, { OP_D,      OP_NUM_SP, OP_NONE }, 1, 1, 1, 0, 9, 16, 9, 26, 9, 24 },
  { "rdlong",  0x0b000000, 0x0fe00000, 2, { OP_D,      OP_NUM_SP, OP_NONE }, 1, 1, 1, 0, 9, 16, 9, 26, 9, 24 },
  { "popa",    0x0b04015f, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 9, 16, 9, 26, 9, 24 },
  { "popb",    0x0b0401df, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 9, 16, 9, 26, 9, 24 },
  { "calld",   0x0b200000, 0x0fe00000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 1, 1, 1, 0, 4, 4, 13, 20, 4, 4 },
  { "resi3",   0x0b3be1f1, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "resi2",   0x0b3be5f3, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "resi1",   0x0b3be9f5, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "resi0",   0x0b3bfdff, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "reti3",   0x0b3bfff1, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "reti2",   0x0b3bfff3, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "reti1",   0x0b3bfff5, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "reti0",   0x0b3bffff, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "callpa",  0x0b400000, 0x0ff00000, 2, { OP_NUM_D,  OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "callpb",  0x0b500000, 0x0ff00000, 2, { OP_NUM_D,  OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "djz",     0x0b600000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "djnz",    0x0b680000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "djf",     0x0b700000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "djnf",    0x0b780000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "ijz",     0x0b800000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "ijnz",    0x0b880000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjz",     0x0b900000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjnz",    0x0b980000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjf",     0x0ba00000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjnf",    0x0ba80000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjs",     0x0bb00000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjns",    0x0bb80000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "tjv",     0x0bc00000, 0x0ff80000, 2, { OP_D,      OP_BRANCH, OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jint",    0x0bc80000, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jct1",    0x0bc80200, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jct2",    0x0bc80400, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jct3",    0x0bc80600, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jse1",    0x0bc80800, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jse2",    0x0bc80a00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jse3",    0x0bc80c00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jse4",    0x0bc80e00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jpat",    0x0bc81000, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jfbw",    0x0bc81200, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jxmt",    0x0bc81400, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jxfi",    0x0bc81600, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jxro",    0x0bc81800, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jxrl",    0x0bc81a00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jatn",    0x0bc81c00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jqmt",    0x0bc81e00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnint",   0x0bc82000, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnct1",   0x0bc82200, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnct2",   0x0bc82400, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnct3",   0x0bc82600, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnse1",   0x0bc82800, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnse2",   0x0bc82a00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnse3",   0x0bc82c00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnse4",   0x0bc82e00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnpat",   0x0bc83000, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnfbw",   0x0bc83200, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnxmt",   0x0bc83400, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnxfi",   0x0bc83600, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnxro",   0x0bc83800, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnxrl",   0x0bc83a00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnatn",   0x0bc83c00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "jnqmt",   0x0bc83e00, 0x0ffbfe00, 1, { OP_BRANCH,  OP_NONE,  OP_NONE }, 0, 0, 0, 0, 2, 4, 2, 20, 2, 4 },
  { "setpat",  0x0bf00000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "akpin",   0x0c080200, 0x0ffbfe00, 1, { OP_NUM_S,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrpin",   0x0c000000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wxpin",   0x0c100000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wypin",   0x0c200000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrlut",   0x0c300000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_SP, OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrbyte",  0x0c400000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_SP, OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "wrword",  0x0c500000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_SP, OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "wrlong",  0x0c600000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_SP, OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "pusha",   0x0c640161, 0x0ff401ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "pushb",   0x0c6401e1, 0x0ff401ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 3, 10, 3, 20, 3, 18 },
  { "rdfast",  0x0c700000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, -1, -1, -1, 2, -1 },
  { "wrfast",  0x0c800000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, -1, -1, -1, 2, -1 },
  { "fblock",  0x0c900000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "xinit",   0x0ca00000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "xstop",   0x0cac0000, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "xzero",   0x0cb00000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, -1, -1, -1, -1, -1, -1 },
  { "xcont",   0x0cc00000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, -1, -1, -1, -1, -1, -1 },
  { "rep",     0x0cd00000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "coginit", 0x0ce00000, 0x0fe00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 1, 0, 0, 2, 11, 2, 11, 2, 19 },
  { "qmul",    0x0d000000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qdiv",    0x0d100000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qfrac",   0x0d200000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qsqrt",   0x0d300000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qrotate", 0x0d400000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qvector", 0x0d500000, 0x0ff00000, 2, { OP_NUM_D,  OP_NUM_S,  OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "hubset",  0x0d600000, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "cogid",   0x0d600001, 0x0fe801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 1, 0, 0, 2, 11, 2, 11, 2, 19 },
  { "cogstop", 0x0d600003, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "locknew", 0x0d600004, 0x0fec01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 1, 0, 0, 4, 11, 4, 11, 4, 19 },
  { "lockret", 0x0d600005, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "locktry", 0x0d600006, 0x0fe801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 1, 0, 0, 2, 11, 2, 11, 2, 19 },
  { "lockrel", 0x0d600007, 0x0fe801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 1, 0, 0, 2, 11, 2, 11, 2, 19 },
  { "qlog",    0x0d60000e, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "qexp",    0x0d60000f, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 9, 2, 9, 2, 17 },
  { "rfbyte",  0x0d600010, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, -1, -1, 2, 2 },
  { "rfword",  0x0d600011, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, -1, -1, 2, 2 },
  { "rflong",  0x0d600012, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, -1, -1, 2, 2 },
  { "rfvar",   0x0d600013, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, -1, -1, 2, 2 },
  { "rfvars",  0x0d600014, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, -1, -1, 2, 2 },
  { "wfbyte",  0x0d600015, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "wfword",  0x0d600016, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "wflong",  0x0d600017, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "getqx",   0x0d600018, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 58, 2, 58, 2, 58 },
  { "getqy",   0x0d600019, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 58, 2, 58, 2, 58 },
  { "getct",   0x0d60001a, 0x0fec01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getrnd",  0x0d60001b, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "getrnd",  0x0d64001b, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "setdacs", 0x0d60001c, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setxfrq", 0x0d60001d, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getxacc", 0x0d60001e, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "waitx",   0x0d60001f, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "setse1",  0x0d600020, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setse2",  0x0d600021, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setse3",  0x0d600022, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setse4",  0x0d600023, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "pollint", 0x0d600024, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollct1", 0x0d600224, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollct2", 0x0d600424, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollct3", 0x0d600624, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollse1", 0x0d600824, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollse2", 0x0d600a24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollse3", 0x0d600c24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollse4", 0x0d600e24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollpat", 0x0d601024, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollfbw", 0x0d601224, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollxmt", 0x0d601424, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollxfi", 0x0d601624, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollxro", 0x0d601824, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollxrl", 0x0d601a24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollatn", 0x0d601c24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "pollqmt", 0x0d601e24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "waitint", 0x0d602024, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitct1", 0x0d602224, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitct2", 0x0d602424, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitct3", 0x0d602624, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitse1", 0x0d602824, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitse2", 0x0d602a24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitse3", 0x0d602c24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitse4", 0x0d602e24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitpat", 0x0d603024, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitfbw", 0x0d603224, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitxmt", 0x0d603424, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitxfi", 0x0d603624, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitxro", 0x0d603824, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitxrl", 0x0d603a24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "waitatn", 0x0d603c24, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, -1, -1, -1, -1, -1, -1 },
  { "allowi",  0x0d604024, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "stalli",  0x0d604224, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "trgint1", 0x0d604424, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "trgint2", 0x0d604624, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "trgint3", 0x0d604824, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "nixint1", 0x0d604a24, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "nixint2", 0x0d604c24, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "nixint3", 0x0d604e24, 0x0fffffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setint1", 0x0d600025, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setint2", 0x0d600026, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setint3", 0x0d600027, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setq",    0x0d600028, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setq2",   0x0d600029, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "push",    0x0d60002a, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "pop",     0x0d60002b, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "jmp",     0x0d60002c, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 4, 4, 13, 20, 4, 4 },
  { "call",    0x0d60002d, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 4, 4, 13, 20, 4, 4 },
  { "ret",     0x0d64002d, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 4, 4, 13, 20, 4, 4 },
  { "calla",   0x0d60002e, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 5, 12, 14, 32, 5, 20 },
  { "reta",    0x0d64002e, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 11, 18, 20, 40, 11, 26 },
  { "callb",   0x0d60002f, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 5, 12, 14, 32, 5, 20 },
  { "retb",    0x0d64002f, 0x0fe7ffff, 0, { OP_NONE,   OP_NONE,   OP_NONE }, 1, 1, 1, 0, 11, 18, 20, 40, 11, 26 },
  { "jmprel",  0x0d600030, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "skip",    0x0d600031, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "skipf",   0x0d600032, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "execf",   0x0d600033, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 4, 4, 4, 4 },
  { "getptr",  0x0d600034, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, -1, -1, 2, 2 },
  { "getbrk",  0x0d600035, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "cogbrk",  0x0d600035, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "brk",     0x0d600036, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setluts", 0x0d600037, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setcy",   0x0d600038, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setci",   0x0d600039, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setcq",   0x0d60003a, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setcfrq", 0x0d60003b, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setcmod", 0x0d60003c, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setpiv",  0x0d60003d, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setpix",  0x0d60003e, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "cogatn",  0x0d60003f, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "testp",   0x0d600040, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "testpn",  0x0d600041, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "testp",   0x0d600042, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 1, 2, 2, 2, 2, 2, 2 },
  { "testpn",  0x0d600043, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 1, 2, 2, 2, 2, 2, 2 },
  { "testp",   0x0d600044, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 2, 2, 2, 2, 2, 2, 2 },
  { "testpn",  0x0d600045, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 2, 2, 2, 2, 2, 2, 2 },
  { "testp",   0x0d600046, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 3, 2, 2, 2, 2, 2, 2 },
  { "testpn",  0x0d600047, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 1, 1, 0, 3, 2, 2, 2, 2, 2, 2 },
  { "dirl",    0x0d600040, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirh",    0x0d600041, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirc",    0x0d600042, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirnc",   0x0d600043, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirz",    0x0d600044, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirnz",   0x0d600045, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirrnd",  0x0d600046, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "dirnot",  0x0d600047, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outl",    0x0d600048, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outh",    0x0d600049, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outc",    0x0d60004a, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outnc",   0x0d60004b, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outz",    0x0d60004c, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outnz",   0x0d60004d, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outrnd",  0x0d60004e, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "outnot",  0x0d60004f, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltl",    0x0d600050, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "flth",    0x0d600051, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltc",    0x0d600052, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltnc",   0x0d600053, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltz",    0x0d600054, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltnz",   0x0d600055, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltrnd",  0x0d600056, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "fltnot",  0x0d600057, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvl",    0x0d600058, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvh",    0x0d600059, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvc",    0x0d60005a, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvnc",   0x0d60005b, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvz",    0x0d60005c, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvnz",   0x0d60005d, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvrnd",  0x0d60005e, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "drvnot",  0x0d60005f, 0x0fe001ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "splitb",  0x0d600060, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "mergeb",  0x0d600061, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "splitw",  0x0d600062, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "mergew",  0x0d600063, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "seussf",  0x0d600064, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "seussr",  0x0d600065, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rgbsqz",  0x0d600066, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rgbexp",  0x0d600067, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "xoro32",  0x0d600068, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rev",     0x0d600069, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "rczr",    0x0d60006a, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "rczl",    0x0d60006b, 0x0fe401ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "wrc",     0x0d60006c, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrnc",    0x0d60006d, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrz",     0x0d60006e, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "wrnz",    0x0d60006f, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "modcz",   0x0d64006f, 0x0fe601ff, 2, { OP_C,      OP_Z,      OP_NONE }, 1, 1, 1, 0, 2, 2, 2, 2, 2, 2 },
  { "modc",    0x0d64006f, 0x0fee1fff, 1, { OP_C,      OP_NONE,   OP_NONE }, 0, 1, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "modz",    0x0d64006f, 0x0ff7e1ff, 1, { OP_Z,      OP_NONE,   OP_NONE }, 1, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "setscp",  0x0d600070, 0x0ff801ff, 1, { OP_NUM_D,  OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "getscp",  0x0d600071, 0x0ffc01ff, 1, { OP_D,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "jmp",     0x0d800000, 0x0fe00000, 1, { OP_A,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "call",    0x0da00000, 0x0fe00000, 1, { OP_A,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "calla",   0x0dc00000, 0x0fe00000, 1, { OP_A,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 5, 12, 14, 32, 5, 20 },
  { "callb",   0x0de00000, 0x0fe00000, 1, { OP_A,      OP_NONE,   OP_NONE }, 0, 0, 0, 0, 5, 12, 14, 32, 5, 20 },
  { "calld",   0x0e000000, 0x0f800000, 2, { OP_P,      OP_A,      OP_NONE }, 0, 0, 0, 0, 4, 4, 13, 20, 4, 4 },
  { "loc",     0x0e800000, 0x0f800000, 2, { OP_P,      OP_A,      OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "augs",    0x0f000000, 0x0f800000, 1, { OP_N_23,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { "augd",    0x0f800000, 0x0f800000, 1, { OP_N_23,   OP_NONE,   OP_NONE }, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2 },
  { NULL }
};

struct _registers_propeller2 registers_propeller2[] =
{
  { "ijmp3", 0x1f0 },
  { "iret3", 0x1f1 },
  { "ijmp2", 0x1f2 },
  { "iret2", 0x1f3 },
  { "ijmp1", 0x1f4 },
  { "iret1", 0x1f5 },
  { "pa",    0x1f6 },
  { "pb",    0x1f7 },
  { "ptra",  0x1f8 },
  { "ptrb",  0x1f9 },
  { "dira",  0x1fa },
  { "dirb",  0x1fb },
  { "outa",  0x1fc },
  { "outb",  0x1fd },
  { "ina",   0x1fe },
  { "inb",   0x1ff },
};

int registers_propeller2_len = sizeof(registers_propeller2) / sizeof(struct _registers_propeller2);

